package de.fhdw.wtf.core.integration.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.AttributeModifier;
import de.fhdw.wtf.common.ast.Constructor;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.OperationModifier;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.ByNameState;
import de.fhdw.wtf.common.ast.type.ClassModifier;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.RegularClassType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.ScannerInputStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.SimpleTokenStream;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.symbols.BracketOpenToken;
import de.fhdw.wtf.common.token.symbols.DoubleSquareBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketOpenToken;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.testutil.TestUtil;

/**
 * Tests the integration of scanner and parser.
 */
public class TestScannerParserIntegration {
	
	private static final String PATH_TO_TEST_FILE_DIR = "src/test/java/de/fhdw/wtf/core/integration/test/files/";
	private static final String MODEL_SUFFIX = "model";
	
	public static ScannerInputStream loadTestModel(final String modelName) throws IOException {
		return TestUtil.readFileToScannerStream(TestScannerParserIntegration.PATH_TO_TEST_FILE_DIR + modelName + '.'
				+ TestScannerParserIntegration.MODEL_SUFFIX);
	}
	
	/**
	 * gruppenName:group=[]; .
	 */
	@Test
	public void modelSimpleValid() {
		final SimpleScannerInput input = new SimpleScannerInput("gruppenName:group=[];");
		final SimpleTokenStream scannerOutput = SimpleTokenStream.create();
		final ModelDslScanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser p = Parser.create(scannerOutput);
		
		try {
			final Model result = p.parse();
			assertEquals(0, p.getExceptions().size());
			assertEquals(1, result.getGroups().size());
			final Group g = result.getGroups().iterator().next();
			assertEquals("gruppenName", ((UnqualifiedName) g.getName()).getIdentifierToken().getIdentifier());
			assertEquals(0, g.getGroupElements().size());
		} catch (final NoValidTokenStreamException e) {
			if (p.getExceptions().size() > 0) {
				System.out.println(p.getExceptions());
			}
			fail();
		}
	}
	
	/**
	 * "gruppenName : group =  \n [    \n];   " .
	 */
	@Test
	public void modelSimpleValidWhitespace() {
		final SimpleScannerInput input = new SimpleScannerInput("gruppenName : group =  \n [	\n];   ");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final ModelDslScanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser p = Parser.create(scannerOutput);
		
		try {
			final Model result = p.parse();
			assertEquals(0, p.getExceptions().size());
			assertEquals(1, result.getGroups().size());
			final Group g = result.getGroups().iterator().next();
			assertEquals("gruppenName", ((UnqualifiedName) g.getName()).getIdentifierToken().getIdentifier());
			assertEquals(0, g.getGroupElements().size());
		} catch (final NoValidTokenStreamException e) {
			if (p.getExceptions().size() > 0) {
				System.out.println(p.getExceptions());
			}
			fail();
		}
	}
	
	/**
	 * Group:group=[Class:class={attribute:[ [key -> value] -> [key -> value] ];};];" .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSquareBracketsInMaps() throws Exception {
		final Position position = Position.create("", 1, 1, 0);
		
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={attribute:[ [key -> value] -> [key -> value] ];};];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final ModelDslScanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser p = Parser.create(scannerOutput);
		final Model actual = p.parse();
		
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", position);
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", position);
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", position);
		final IdentifierToken keyIdentifier = IdentifierToken.create("key", position);
		final IdentifierToken valueIdentifier = IdentifierToken.create("value", position);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						new Vector<ClassType>());
		
		final ByNameState keyReferenceState = ByNameState.create(UnqualifiedName.create(keyIdentifier));
		final TypeProxy keyType = TypeProxy.create(keyIdentifier, keyReferenceState);
		final ByNameState valueReferenceState = ByNameState.create(UnqualifiedName.create(valueIdentifier));
		final TypeProxy valueType = TypeProxy.create(valueIdentifier, valueReferenceState);
		
		final MapType mapIn1 = MapType.create(SquareBracketOpenToken.create(position), keyType, valueType);
		final MapType mapIn2 = MapType.create(SquareBracketOpenToken.create(position), keyType, valueType);
		final MapType mapOut = MapType.create(SquareBracketOpenToken.create(position), mapIn1, mapIn2);
		
		final Attribute attribute =
				Attribute.create("attribute", mapOut, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, actual);
		assertEquals(0, p.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={operation:[[(parameter1) -> [key-> [key->value] ] ]];}] .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSquareBracketsInOperation() throws Exception {
		final Position position = Position.create("", 1, 1, 0);
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Class:class={operation:[[(parameter:String) -> [key-> [key->value] ] ]];};];");
		final FilteredTokenStream scannerOutput = FilteredTokenStream.create();
		final ModelDslScanner scanner = ModelDslScanner.create();
		scanner.scan(input, scannerOutput);
		final Parser p = Parser.create(scannerOutput);
		final Model actual = p.parse();
		
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", position);
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", position);
		final IdentifierToken keyIdentifier = IdentifierToken.create("key", position);
		final IdentifierToken valueIdentifier = IdentifierToken.create("value", position);
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", position);
		final IdentifierToken parameterIdentifier = IdentifierToken.create("parameter", position);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						new Vector<Constructor>(),
						classIdentifier,
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(stringIdentifier, stringReferenceState);
		final ByNameState keyReferenceState = ByNameState.create(UnqualifiedName.create(keyIdentifier));
		final TypeProxy keyType = TypeProxy.create(keyIdentifier, keyReferenceState);
		final ByNameState valueReferenceState = ByNameState.create(UnqualifiedName.create(valueIdentifier));
		final TypeProxy valueType = TypeProxy.create(valueIdentifier, valueReferenceState);
		
		final ProductElementType element = ProductElementType.create("parameter", stringType, parameterIdentifier);
		final ProductType product = ProductType.create(BracketOpenToken.create(position));
		product.addElement(element);
		
		final MapType mapIn = MapType.create(SquareBracketOpenToken.create(position), keyType, valueType);
		final MapType mapOut = MapType.create(SquareBracketOpenToken.create(position), keyType, mapIn);
		
		final Operation op =
				Operation.create(
						"operation",
						new Vector<OperationModifier>(),
						product,
						classModel,
						mapOut,
						DoubleSquareBracketOpenToken.create(position));
		operations.add(op);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, actual);
		assertEquals(0, p.getExceptions().size());
	}
	
	/**
	 * Tests the model "Person".
	 * 
	 * @throws IOException
	 */
	@Test
	public void modelFilePerson() throws IOException {
		System.out.println(new File("").getAbsolutePath());
		final ScannerInputStream input = TestScannerParserIntegration.loadTestModel("Person");
		final FilteredTokenStream output = FilteredTokenStream.create();
		final ModelDslScanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser p = Parser.create(output);
		
		try {
			final Model result = p.parse();
			assertEquals(0, p.getExceptions().size());
			assertEquals(1, result.getGroups().size());
			final Group g = result.getGroups().iterator().next();
			assertEquals("personGroup", ((UnqualifiedName) g.getName()).getIdentifierToken().getIdentifier());
			assertEquals(3, g.getGroupElements().size());
		} catch (final NoValidTokenStreamException e) {
			if (p.getExceptions().size() > 0) {
				System.out.println(p.getExceptions());
			}
			fail();
		}
		
	}
	
}
