package de.fhdw.wtf.parser.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Iterator;
import java.util.Vector;

import org.junit.Before;
import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.Constructor;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.ByNameState;
import de.fhdw.wtf.common.ast.type.ClassModifier;
import de.fhdw.wtf.common.ast.type.ClassModifierAbstract;
import de.fhdw.wtf.common.ast.type.ClassModifierService;
import de.fhdw.wtf.common.ast.type.ClassModifierTransient;
import de.fhdw.wtf.common.ast.type.ClassModifierVisitable;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.RegularClassType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.exception.parser.AbstractParserException;
import de.fhdw.wtf.common.exception.parser.ClassModifierAlreadyAddedException;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.stream.SimpleTokenStream;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.EndToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.keywords.AbstractToken;
import de.fhdw.wtf.common.token.keywords.ClassToken;
import de.fhdw.wtf.common.token.keywords.GroupToken;
import de.fhdw.wtf.common.token.keywords.ServiceToken;
import de.fhdw.wtf.common.token.keywords.TransientToken;
import de.fhdw.wtf.common.token.keywords.VisitableToken;
import de.fhdw.wtf.common.token.symbols.ColonToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.EqualToken;
import de.fhdw.wtf.common.token.symbols.PlusSymbolToken;
import de.fhdw.wtf.common.token.symbols.SemicolonToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketOpenToken;
import de.fhdw.wtf.parser.Parser;

/**
 * Tests the parsing of classes.
 */
public class TestClasses {
	
	private Parser parser;
	private TokenStream stream;
	
	private IdentifierToken group1IdentifierToken;
	private IdentifierToken group2IdentifierToken;
	
	private IdentifierToken classIdentifierToken;
	private IdentifierToken superClass1IdentifierToken;
	private IdentifierToken superClass2IdentifierToken;
	
	private ColonToken colonToken;
	private GroupToken groupToken;
	private EqualToken equalToken;
	private SquareBracketOpenToken squareBracketOpenToken;
	private SquareBracketCloseToken squareBracketCloseToken;
	private SemicolonToken semicolonToken;
	private ClassToken classToken;
	private CurlyBracketOpenToken curlyBracketOpenToken;
	private CurlyBracketCloseToken curlyBracketCloseToken;
	private EndToken endToken;
	private AbstractToken abstractToken;
	private TransientToken transientToken;
	private ServiceToken serviceToken;
	private VisitableToken visitableToken;
	private PlusSymbolToken plusSymbolToken;
	
	/**
	 * Creates the most important Tokens.
	 * 
	 * @throws Exception
	 */
	@Before
	public void setUp() throws Exception {
		this.stream = SimpleTokenStream.create();
		this.parser = Parser.create(this.stream);
		
		this.group1IdentifierToken = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		this.group2IdentifierToken = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		this.classIdentifierToken = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		this.superClass1IdentifierToken = IdentifierToken.create("Superclass", Position.create("", 0 + 1, 0 + 1, 0));
		this.superClass2IdentifierToken = IdentifierToken.create("Superclass", Position.create("", 0 + 1, 0 + 1, 0));
		this.colonToken = ColonToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.groupToken = GroupToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.equalToken = EqualToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.squareBracketOpenToken = SquareBracketOpenToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.squareBracketCloseToken = SquareBracketCloseToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.semicolonToken = SemicolonToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.classToken = ClassToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.curlyBracketOpenToken = CurlyBracketOpenToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.curlyBracketCloseToken = CurlyBracketCloseToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.endToken = EndToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.abstractToken = AbstractToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.transientToken = TransientToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.serviceToken = ServiceToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.visitableToken = VisitableToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.plusSymbolToken = PlusSymbolToken.create(Position.create("", 0 + 1, 0 + 1, 0));
	}
	
	/**
	 * Group:group=[Class:class={};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleClass() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, this.group1IdentifierToken);
		final Name className = groupName.addName(this.classIdentifierToken);
		final ClassType class1 =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
		
	}
	
	/**
	 * Group:group=[Class:class={}]; .
	 */
	@Test
	public void testSimpleClassWithoutSemicolon() {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
		}
	}
	
	/**
	 * Group:group=[Class:class={;]; .
	 */
	@Test
	public void testSimpleClassWithoutClassEnd() {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
		}
	}
	
	/**
	 * Class:class={}; .
	 */
	@Test
	public void testClassWithoutAGroup() {
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(EndToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
		}
	}
	
	/**
	 * Group:group=[Class:class={} abstract;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testClassAbstractModifier() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, this.group1IdentifierToken);
		final Name className = groupName.addName(this.classIdentifierToken);
		modifiers.add(ClassModifierAbstract.create(this.abstractToken));
		final ClassType class1 =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={} transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testClassTransientModifier() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.transientToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, this.group1IdentifierToken);
		final Name className = groupName.addName(this.classIdentifierToken);
		modifiers.add(ClassModifierTransient.create(this.transientToken));
		final ClassType class1 =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={} service;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testClassServiceModifier() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.serviceToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, this.group1IdentifierToken);
		final Name className = groupName.addName(this.classIdentifierToken);
		modifiers.add(ClassModifierService.create(this.serviceToken));
		final ClassType class1 =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={} visitable;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testClassVisitableModifier() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.visitableToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, this.group1IdentifierToken);
		final Name className = groupName.addName(this.classIdentifierToken);
		modifiers.add(ClassModifierVisitable.create(this.visitableToken));
		final ClassType class1 =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={} [;]; .
	 */
	@Test
	public void testClassWithExceptionInParseModifier() {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
		}
	}
	
	/**
	 * Group:group=[Class:class={} visitable visitable;]; .
	 */
	@Test
	public void testClassWithMultipleSameModifiers() {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.visitableToken);
		this.stream.add(this.visitableToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
		}
	}
	
	/**
	 * Group:group=[Class:class={} abstract transient service visitable;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testClassWithAllModifier() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.transientToken);
		this.stream.add(this.serviceToken);
		this.stream.add(this.visitableToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, this.group1IdentifierToken);
		final Name className = groupName.addName(this.classIdentifierToken);
		modifiers.add(ClassModifierAbstract.create(this.abstractToken));
		modifiers.add(ClassModifierTransient.create(this.transientToken));
		modifiers.add(ClassModifierService.create(this.serviceToken));
		modifiers.add(ClassModifierVisitable.create(this.visitableToken));
		final ClassType class1 =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={} service abstract visitable transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testClassWithAllModifierInDifferentOrder() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.serviceToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.visitableToken);
		this.stream.add(this.transientToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, this.group1IdentifierToken);
		final Name className = groupName.addName(this.classIdentifierToken);
		modifiers.add(ClassModifierService.create(this.serviceToken));
		modifiers.add(ClassModifierAbstract.create(this.abstractToken));
		modifiers.add(ClassModifierVisitable.create(this.visitableToken));
		modifiers.add(ClassModifierTransient.create(this.transientToken));
		final ClassType class1 =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={} abstract abstract abstract abstract abstract;]; .
	 */
	@Test
	public void testClassMultipleAbstractModifier() {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.abstractToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(4, this.parser.getExceptions().size());
			final Iterator<AbstractParserException> i = this.parser.getExceptions().iterator();
			while (i.hasNext()) {
				final AbstractParserException current = i.next();
				assertTrue(current instanceof ClassModifierAlreadyAddedException);
			}
		}
	}
	
	/**
	 * Group:group=[Superclass:class={};Class:class=Superclass+{};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleInheritance() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.superClass1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.superClass1IdentifierToken);
		this.stream.add(this.plusSymbolToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, this.group1IdentifierToken);
		final Name className = groupName.addName(this.classIdentifierToken);
		final Name superclassName = groupName.addName(this.superClass1IdentifierToken);
		final TypeProxy supertype =
				TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(this.superClass1IdentifierToken)));
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(supertype);
		final ClassType class1 =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		
		final ClassType superclass =
				RegularClassType.create(
						superclassName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.superClass1IdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(superclass);
		groupVector.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Superclass1:class={};Superclass2:class{};Subclass:class= Superclass1+Superclass2+{};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testMultipleInheritance() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.superClass1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.superClass2IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.superClass1IdentifierToken);
		this.stream.add(this.plusSymbolToken);
		this.stream.add(this.superClass2IdentifierToken);
		this.stream.add(this.plusSymbolToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group = Group.create(groupName, groupVector, this.group1IdentifierToken);
		final Name className = groupName.addName(this.classIdentifierToken);
		
		final TypeProxy supertype =
				TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(this.superClass1IdentifierToken)));
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(supertype);
		supertypes.add(supertype);
		
		final ClassType class1 =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		final Name superclass1Name = groupName.addName(this.superClass1IdentifierToken);
		final ClassType superclass1 =
				RegularClassType.create(
						superclass1Name,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.superClass1IdentifierToken,
						
						new Vector<ClassType>());
		final Name superclass2Name = groupName.addName(this.superClass2IdentifierToken);
		final ClassType superclass2 =
				RegularClassType.create(
						superclass2Name,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.superClass1IdentifierToken,
						
						new Vector<ClassType>());
		groupVector.add(superclass1);
		groupVector.add(superclass2);
		groupVector.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group1:group=[Superclass:class={};]; Group2:group=[Class:class=Superclass+{};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleInheritanceWithTwoGroups() throws Exception {
		this.stream.add(this.group1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.superClass1IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.group2IdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.classIdentifierToken);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.superClass1IdentifierToken);
		this.stream.add(this.plusSymbolToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector1 = new Vector<>();
		final Vector<GroupElement> groupVector2 = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(this.group1IdentifierToken);
		final Group group1 = Group.create(group1Name, groupVector1, this.group1IdentifierToken);
		
		final Name group2Name = UnqualifiedName.create(this.group2IdentifierToken);
		final Group group2 = Group.create(group2Name, groupVector2, this.group2IdentifierToken);
		
		final TypeProxy supertype =
				TypeProxy.create(null, ByNameState.create(UnqualifiedName.create(this.superClass1IdentifierToken)));
		final Vector<Type> supertypes = new Vector<>();
		supertypes.add(supertype);
		
		final Name className = group2Name.addName(this.classIdentifierToken);
		final ClassType class1 =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						supertypes,
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.classIdentifierToken,
						
						new Vector<ClassType>());
		final Name superclassName = group1Name.addName(this.superClass1IdentifierToken);
		final ClassType superclass =
				RegularClassType.create(
						superclassName,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						this.superClass1IdentifierToken,
						
						new Vector<ClassType>());
		
		groupVector1.add(superclass);
		groupVector2.add(class1);
		final Model expected = Model.create(this.group1IdentifierToken);
		expected.addGroup(group1);
		expected.addGroup(group2);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
}
