package de.fhdw.wtf.parser.test;

import static org.junit.Assert.assertEquals;

import java.util.Collection;

import org.junit.Before;
import org.junit.Test;

import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Tuple;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.stream.SimpleTokenStream;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.EndToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.keywords.ClassToken;
import de.fhdw.wtf.common.token.keywords.GroupToken;
import de.fhdw.wtf.common.token.symbols.ColonToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.EqualToken;
import de.fhdw.wtf.common.token.symbols.SemicolonToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketOpenToken;
import de.fhdw.wtf.parser.Parser;

/**
 * Tests functions which are used to work with a parsed {@link Model}.
 */
public class TestModels {
	
	/**
	 * The {@link ColonToken} represents ":".
	 */
	private ColonToken colonToken;
	
	/**
	 * The {@link GroupToken} represents "group".
	 */
	private GroupToken groupToken;
	
	/**
	 * The {@link EqualToken} represents "=".
	 */
	private EqualToken equalToken;
	
	/**
	 * The {@link SquareBracketOpenToken} represents "[".
	 */
	private SquareBracketOpenToken squareBracketOpenToken;
	
	/**
	 * The {@link SquareBracketCloseToken} represents "]".
	 */
	private SquareBracketCloseToken squareBracketCloseToken;
	
	/**
	 * The {@link SemicolonToken} represents ";".
	 */
	private SemicolonToken semicolonToken;
	
	/**
	 * The {@link ClassToken} represents "class".
	 */
	private ClassToken classToken;
	
	/**
	 * The {@link CurlyBracketOpenToken} represents "{".
	 */
	private CurlyBracketOpenToken curlyBracketOpenToken;
	
	/**
	 * The {@link CurlyBracketCloseToken} represents "}".
	 */
	private CurlyBracketCloseToken curlyBracketCloseToken;
	
	/**
	 * The {@link EndToken} represents "".
	 */
	private EndToken endToken;
	
	/**
	 * Creates the most important Tokens.
	 */
	@Before
	public void setUp() {
		this.colonToken = ColonToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.groupToken = GroupToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.equalToken = EqualToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.squareBracketOpenToken = SquareBracketOpenToken.create(Position.create("", 0, 0, 0));
		this.squareBracketCloseToken = SquareBracketCloseToken.create(Position.create("", 0, 0, 0));
		this.semicolonToken = SemicolonToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.classToken = ClassToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.curlyBracketOpenToken = CurlyBracketOpenToken.create(Position.create("", 0, 0, 0));
		this.curlyBracketCloseToken = CurlyBracketCloseToken.create(Position.create("", 0, 0, 0));
		this.endToken = EndToken.create(Position.create("", 0 + 1, 0 + 1, 0));
	}
	
	/**
	 * Group1:group=[Class1:class={};]; Group2:group=[Class2:class={};Class3:class={}];.
	 * 
	 * @throws Exception
	 *             : {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testModelCalculation() throws Exception {
		final TokenStream stream = SimpleTokenStream.create();
		final Parser parser = Parser.create(stream);
		final IdentifierToken group1Identifier = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken class1Identifier = IdentifierToken.create("Class1", Position.create("", 1, 1, 0));
		final IdentifierToken group2Identifier = IdentifierToken.create("Group2", Position.create("", 1, 1, 0));
		final IdentifierToken class2Identifier = IdentifierToken.create("Class2", Position.create("", 1, 1, 0));
		final IdentifierToken class3Identifier = IdentifierToken.create("Class3", Position.create("", 1, 1, 0));
		
		stream.add(group1Identifier);
		stream.add(this.colonToken);
		stream.add(this.groupToken);
		stream.add(this.equalToken);
		stream.add(this.squareBracketOpenToken);
		stream.add(class1Identifier);
		stream.add(this.colonToken);
		stream.add(this.classToken);
		stream.add(this.equalToken);
		stream.add(this.curlyBracketOpenToken);
		stream.add(this.curlyBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(this.squareBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(group2Identifier);
		stream.add(this.colonToken);
		stream.add(this.groupToken);
		stream.add(this.equalToken);
		stream.add(this.squareBracketOpenToken);
		stream.add(class2Identifier);
		stream.add(this.colonToken);
		stream.add(this.classToken);
		stream.add(this.equalToken);
		stream.add(this.curlyBracketOpenToken);
		stream.add(this.curlyBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(class3Identifier);
		stream.add(this.colonToken);
		stream.add(this.classToken);
		stream.add(this.equalToken);
		stream.add(this.curlyBracketOpenToken);
		stream.add(this.curlyBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(this.squareBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(this.endToken);
		
		final Model model = parser.parse();
		final Tuple<Collection<ClassType>, Collection<Group>> groupsClasses = model.calcAllGroupsAndClasses();
		assertEquals(3, groupsClasses.getFirst().size());
		assertEquals(2, groupsClasses.getSecond().size());
	}
	
	/**
	 * Group1:group=[Group3:group=[Class4:class={};]; Class1:class={};];
	 * Group2:group=[Class2:class={};Class3:class={}];.
	 * 
	 * @throws Exception
	 *             : {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testModelCalculation2() throws Exception {
		final TokenStream stream = SimpleTokenStream.create();
		final Parser parser = Parser.create(stream);
		final IdentifierToken group1Identifier = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken class1Identifier = IdentifierToken.create("Class1", Position.create("", 1, 1, 0));
		final IdentifierToken group2Identifier = IdentifierToken.create("Group2", Position.create("", 1, 1, 0));
		final IdentifierToken class2Identifier = IdentifierToken.create("Class2", Position.create("", 1, 1, 0));
		final IdentifierToken class3Identifier = IdentifierToken.create("Class3", Position.create("", 1, 1, 0));
		final IdentifierToken group3Identifier = IdentifierToken.create("Group3", Position.create("", 1, 1, 0));
		final IdentifierToken class4Identifier = IdentifierToken.create("Class4", Position.create("", 1, 1, 0));
		
		stream.add(group1Identifier);
		stream.add(this.colonToken);
		stream.add(this.groupToken);
		stream.add(this.equalToken);
		stream.add(this.squareBracketOpenToken);
		stream.add(group3Identifier);
		stream.add(this.colonToken);
		stream.add(this.groupToken);
		stream.add(this.equalToken);
		stream.add(this.squareBracketOpenToken);
		stream.add(class4Identifier);
		stream.add(this.colonToken);
		stream.add(this.classToken);
		stream.add(this.equalToken);
		stream.add(this.curlyBracketOpenToken);
		stream.add(this.curlyBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(this.squareBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(class1Identifier);
		stream.add(this.colonToken);
		stream.add(this.classToken);
		stream.add(this.equalToken);
		stream.add(this.curlyBracketOpenToken);
		stream.add(this.curlyBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(this.squareBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(group2Identifier);
		stream.add(this.colonToken);
		stream.add(this.groupToken);
		stream.add(this.equalToken);
		stream.add(this.squareBracketOpenToken);
		stream.add(class2Identifier);
		stream.add(this.colonToken);
		stream.add(this.classToken);
		stream.add(this.equalToken);
		stream.add(this.curlyBracketOpenToken);
		stream.add(this.curlyBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(class3Identifier);
		stream.add(this.colonToken);
		stream.add(this.classToken);
		stream.add(this.equalToken);
		stream.add(this.curlyBracketOpenToken);
		stream.add(this.curlyBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(this.squareBracketCloseToken);
		stream.add(this.semicolonToken);
		stream.add(this.endToken);
		
		final Model model = parser.parse();
		final Tuple<Collection<ClassType>, Collection<Group>> groupsClasses = model.calcAllGroupsAndClasses();
		assertEquals(4, groupsClasses.getFirst().size());
		assertEquals(3, groupsClasses.getSecond().size());
	}
	
	/**
	 * Group1:group=[Class1:class={};]; Group2:group=[Class2:class={};Class3:class={}];.
	 * 
	 * @throws Exception
	 *             : {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}
	 */
	@Test
	public void testModelHashCode() throws Exception {
		final TokenStream stream1 = SimpleTokenStream.create();
		final TokenStream stream2 = SimpleTokenStream.create();
		final Parser parser1 = Parser.create(stream1);
		final Parser parser2 = Parser.create(stream2);
		final IdentifierToken group1Identifier = IdentifierToken.create("Group1", Position.create("", 1, 1, 0));
		final IdentifierToken class1Identifier = IdentifierToken.create("Class1", Position.create("", 1, 1, 0));
		final IdentifierToken group2Identifier = IdentifierToken.create("Group2", Position.create("", 1, 1, 0));
		final IdentifierToken class2Identifier = IdentifierToken.create("Class2", Position.create("", 1, 1, 0));
		final IdentifierToken class3Identifier = IdentifierToken.create("Class3", Position.create("", 1, 1, 0));
		stream1.add(group1Identifier);
		stream1.add(this.colonToken);
		stream1.add(this.groupToken);
		stream1.add(this.equalToken);
		stream1.add(this.squareBracketOpenToken);
		stream1.add(class1Identifier);
		stream1.add(this.colonToken);
		stream1.add(this.classToken);
		stream1.add(this.equalToken);
		stream1.add(this.curlyBracketOpenToken);
		stream1.add(this.curlyBracketCloseToken);
		stream1.add(this.semicolonToken);
		stream1.add(this.squareBracketCloseToken);
		stream1.add(this.semicolonToken);
		stream1.add(group2Identifier);
		stream1.add(this.colonToken);
		stream1.add(this.groupToken);
		stream1.add(this.equalToken);
		stream1.add(this.squareBracketOpenToken);
		stream1.add(class2Identifier);
		stream1.add(this.colonToken);
		stream1.add(this.classToken);
		stream1.add(this.equalToken);
		stream1.add(this.curlyBracketOpenToken);
		stream1.add(this.curlyBracketCloseToken);
		stream1.add(this.semicolonToken);
		stream1.add(class3Identifier);
		stream1.add(this.colonToken);
		stream1.add(this.classToken);
		stream1.add(this.equalToken);
		stream1.add(this.curlyBracketOpenToken);
		stream1.add(this.curlyBracketCloseToken);
		stream1.add(this.semicolonToken);
		stream1.add(this.squareBracketCloseToken);
		stream1.add(this.semicolonToken);
		stream1.add(this.endToken);
		
		final Model model1 = parser1.parse();
		
		stream2.add(group1Identifier);
		stream2.add(this.colonToken);
		stream2.add(this.groupToken);
		stream2.add(this.equalToken);
		stream2.add(this.squareBracketOpenToken);
		stream2.add(class1Identifier);
		stream2.add(this.colonToken);
		stream2.add(this.classToken);
		stream2.add(this.equalToken);
		stream2.add(this.curlyBracketOpenToken);
		stream2.add(this.curlyBracketCloseToken);
		stream2.add(this.semicolonToken);
		stream2.add(this.squareBracketCloseToken);
		stream2.add(this.semicolonToken);
		stream2.add(group2Identifier);
		stream2.add(this.colonToken);
		stream2.add(this.groupToken);
		stream2.add(this.equalToken);
		stream2.add(this.squareBracketOpenToken);
		stream2.add(class2Identifier);
		stream2.add(this.colonToken);
		stream2.add(this.classToken);
		stream2.add(this.equalToken);
		stream2.add(this.curlyBracketOpenToken);
		stream2.add(this.curlyBracketCloseToken);
		stream2.add(this.semicolonToken);
		stream2.add(class3Identifier);
		stream2.add(this.colonToken);
		stream2.add(this.classToken);
		stream2.add(this.equalToken);
		stream2.add(this.curlyBracketOpenToken);
		stream2.add(this.curlyBracketCloseToken);
		stream2.add(this.semicolonToken);
		stream2.add(this.squareBracketCloseToken);
		stream2.add(this.semicolonToken);
		stream2.add(this.endToken);
		
		final Model model2 = parser2.parse();
		assertEquals(model1, model2);
		assertEquals(model1.hashCode(), model2.hashCode());
	}
}
