package de.fhdw.wtf.walker.tasks.test;

import java.util.Collection;
import java.util.Vector;
import java.util.concurrent.ExecutionException;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.DoubleGroupcomponentCheck;
import de.fhdw.wtf.walker.walker.SimpleWalkerTask;

public class TestDoubleGroupcomponentCheck extends TestCase {
	
	/**
	 * Checks if 2 of the same classnames will through a error.
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 */
	@Test
	public void testDoubleGroupcomponent1_1() throws InterruptedException, ExecutionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("" + "group1:group=[" + "class1:class={};" + "class1:class={};"
						+ "group2:group=[" + "class2:class={};" + "group3:group=[" + "class3:class={"
						+ "a:group2>class2;" + "};" + "];" + "];" + "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		try {
			model = parser.parse();
		} catch (final NoValidTokenStreamException e) {
			fail();
		}
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleClassname = DoubleGroupcomponentCheck.create(model, taskmanager);
		
		final Collection<SimpleWalkerTask> tasks = new Vector<>();
		tasks.add(doubleClassname);
		taskmanager.start(tasks);
		
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			assertTrue(current instanceof ExceptionalTaskResult);
		}
		assertEquals(1, results.size());
	}
	
	/**
	 * Checks if 2 of the same classnames will through a error if there in different groups (shouldnt).
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 */
	@Test
	public void testDoubleGroupcomponent1_2() throws InterruptedException, ExecutionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("" + "group1:group=[" + "class1:class={};" + "];" + "group2:group=["
						+ "class1:class={};" + "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		try {
			model = parser.parse();
		} catch (final NoValidTokenStreamException e) {
			fail();
		}
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleClassname = DoubleGroupcomponentCheck.create(model, taskmanager);
		
		final Collection<SimpleWalkerTask> tasks = new Vector<>();
		tasks.add(doubleClassname);
		taskmanager.start(tasks);
		
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			assertFalse(current instanceof ExceptionalTaskResult);
		}
		assertEquals(1, results.size());
	}
	
	/**
	 * checks wether 2 groupnames can have the same name.
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 */
	@Test
	public void testDoubleGroupcomponent2() throws InterruptedException, ExecutionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("" + "group1:group=[" + "class1:class={};" + "group2:group=[];"
						+ "group2:group=[" + "class2:class={};" + "group3:group=[" + "class3:class={"
						+ "a:group2>class2;" + "};" + "];" + "];" + "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		try {
			model = parser.parse();
		} catch (final NoValidTokenStreamException e) {
			fail();
		}
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		
		final Collection<SimpleWalkerTask> tasks = new Vector<>();
		tasks.add(doubleGroupcomponent);
		taskmanager.start(tasks);
		
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			assertTrue(current instanceof ExceptionalTaskResult);
		}
		assertEquals(1, results.size());
	}
	
	/**
	 * checks wether a group can have the same name as a class.
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 */
	@Test
	public void testDoubleGroupcomponents3() throws InterruptedException, ExecutionException {
		final SimpleScannerInput input = new SimpleScannerInput("" + "k:group=[" + "k:class={};" + "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		try {
			model = parser.parse();
		} catch (final NoValidTokenStreamException e) {
			fail();
		}
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		
		final Collection<SimpleWalkerTask> tasks = new Vector<>();
		tasks.add(doubleGroupcomponent);
		taskmanager.start(tasks);
		
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			assertFalse(current instanceof ExceptionalTaskResult);
		}
		assertEquals(1, results.size());
	}
	
	/**
	 * checks whether a group can have the same name as a class when the class is in same group.
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 */
	@Test
	public void testDoubleGroupcomponents4() throws InterruptedException, ExecutionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("" + "group1:group=[" + "k:class={};" + "k:group=[];" + "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		try {
			model = parser.parse();
		} catch (final NoValidTokenStreamException e) {
			fail();
		}
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		
		final Collection<SimpleWalkerTask> tasks = new Vector<>();
		tasks.add(doubleGroupcomponent);
		taskmanager.start(tasks);
		
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			assertTrue(current instanceof ExceptionalTaskResult);
		}
		assertEquals(1, results.size());
	}
	
	/**
	 * checks whether a group can have the same name as a class when the class is not in same group (shouldnt error).
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 */
	@Test
	public void testDoubleGroupcomponents5() throws InterruptedException, ExecutionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("" + "group1:group=[" + "k:class={};" + "];" + "group2:group=[" + "k:group=[];"
						+ "];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		try {
			model = parser.parse();
		} catch (final NoValidTokenStreamException e) {
			fail();
		}
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		
		final Collection<SimpleWalkerTask> tasks = new Vector<>();
		tasks.add(doubleGroupcomponent);
		taskmanager.start(tasks);
		
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			assertFalse(current instanceof ExceptionalTaskResult);
		}
		assertEquals(1, results.size());
	}
	
}
