package de.fhdw.wtf.walker.tasks.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.ByReferenceState;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.constants.referencer.BaseTypeConstants;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.exception.walker.CyclicDependencyException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.OKTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.common.task.result.visitor.TaskResultVisitor;
import de.fhdw.wtf.common.token.DummyToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.CyclicInheritanceCheck;
import de.fhdw.wtf.walker.tasks.PrototypesTask;
import de.fhdw.wtf.walker.tasks.SubtypesFillTask;
import de.fhdw.wtf.walker.tasks.SupertypesFillTask;
import de.fhdw.wtf.walker.tasks.TypeReferencer;
import de.fhdw.wtf.walker.walker.HelperUtils;
import de.fhdw.wtf.walker.walker.SimpleWalkerTask;
import de.fhdw.wtf.walker.walker.SimpleWalkerTaskForTypes;

/**
 * Tests {@link de.fhdw.wtf.walker.tasks.SumSupertypesFillTask}.
 */
public class TestSupertypesFillTask {
	
	/**
	 * Entsprechende Task laufen und erzeugen das Model.
	 * 
	 * @param input
	 *            SimpleScannerInput
	 * @return Model
	 * @throws CyclicDependencyException
	 *             CyclicDependencyException
	 * @throws InterruptedException
	 *             InterruptedException
	 * @throws ExecutionException
	 *             ExecutionException
	 * @throws NoValidTokenStreamException
	 *             NoValidTokenStreamException
	 */
	private Model getModel(final SimpleScannerInput input) throws CyclicDependencyException, InterruptedException,
			ExecutionException, NoValidTokenStreamException {
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		assertEquals(0, parser.getExceptions().size());
		
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		final TypeReferencer referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTaskForTypes supertypesFillTask = SupertypesFillTask.create(model, taskmanager);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		supertypesFillTask.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(5, okResult.size());
		assertEquals(0, failResult.size());
		
		return model;
	}
	
	/**
	 * Group:group=[Class:class={Attribute:String;};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testModelWithoutSum() throws Exception {
		final SimpleScannerInput input = new SimpleScannerInput("Group:group=[Class:class={Attribute:String;};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		
		assertEquals(1, clss.getSuperTypes().size());
		assertEquals(1, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{String,Integer}; Attribute2:{String};};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testNormalContainsInheritance() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute1:{String,Integer,Class}; "
						+ "Attribute2:{String,Integer};};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		final Iterator<Attribute> attrIterator = clss.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		assertTrue(attr1Type == model.getAnything().getPrototype());
		assertEquals(0, supertypesAttr1.size());
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		assertEquals(1, supertypesAttr2.size());
		final Iterator<Type> superTypesAttr2Iterator = supertypesAttr2.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr2Iterator.next()));
		// {String,Integer,Class} == Anything !
		assertEquals(1, clss.getSuperTypes().size());
		// {String,Integer}, Anything
		assertEquals(2, model.getString().getSuperTypes().size());
		// {String,Integer}, Anything
		assertEquals(2, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:{String,Integer}; Attribute2:{String};}; Class2:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testNormalContainsInheritance2() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Class:class={Attribute1:{String,Integer,Class}; "
						+ "Attribute2:{String,Integer};};" + "Class2:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		final Iterator<Attribute> attrIterator = clss.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		assertEquals(1, supertypesAttr1.size());
		final Iterator<Type> superTypesAttr1Iterator = supertypesAttr1.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr1Iterator.next()));
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		assertEquals(2, supertypesAttr2.size());
		final Iterator<Type> superTypesAttr2Iterator = supertypesAttr2.iterator();
		assertEquals(attr1Type, superTypesAttr2Iterator.next());
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr2Iterator.next()));
		
		// {String,Integer,Class} != Anything !
		assertEquals(2, clss.getSuperTypes().size());
		// {String,Integer,Class}, {String,Integer}, Anything
		assertEquals(3, model.getString().getSuperTypes().size());
		// {String,Integer,Class}, {String,Integer}, Anything
		assertEquals(3, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{B,C};}; B:class=A+{}; C:class=A+{};
	 * Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testContainsInheritanceOverClassInheritanceAndSumSupertypeIsAnClass() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{B,C};}; "
						+ "B:class=A+{}; C:class=A+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		final Iterator<Attribute> attrIterator = clss.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		assertEquals(1, supertypesAttr1.size());
		final Iterator<Type> superTypesAttr1Iterator = supertypesAttr1.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr1Iterator.next()));
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		
		final Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(attr1Type);
		expectedSupertypes.add(clss);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr2);
		
		assertEquals(2, clss.getSuperTypes().size());
		assertEquals(2, model.getString().getSuperTypes().size());
		assertEquals(2, model.getInteger().getSuperTypes().size());
		
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{String,C};}; B:class=A+{}; C:class=A+{};
	 * Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testContainsInheritanceOverClassInheritance2() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{String,C};}; "
						+ "B:class=A+{}; C:class=A+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		final Iterator<Attribute> attrIterator = clss.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		assertEquals(1, supertypesAttr1.size());
		final Iterator<Type> superTypesAttr1Iterator = supertypesAttr1.iterator();
		final Type supertype = superTypesAttr1Iterator.next();
		final ByReferenceState state =
				ByReferenceState.create(
						model.getAnything().getPrototype(),
						UnqualifiedName.create(IdentifierToken.create(
								BaseTypeConstants.ANYTHING_CONSTANT,
								DummyToken.getDummyPosition())));
		final TypeProxy typeProxy = TypeProxy.create(DummyToken.getInstance(), state, DummyToken.getInstance());
		assertEquals(typeProxy, supertype);
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(supertype));
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		
		final Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(attr1Type);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr2);
		
		assertEquals(2, clss.getSuperTypes().size());
		assertEquals(3, model.getString().getSuperTypes().size());
		assertEquals(2, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{A,B,C}; Attribute2:{A,B}; Attribute3:{B,C}; Attribute4:{A,C};}; B:class=A+{};
	 * C:class=A+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testNormalContainsInheritance3() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={Attribute1:{A,B,C}; Attribute2:{A,B}; Attribute3:{B,C}; "
						+ "Attribute4:{A,C};}; B:class={}; C:class={}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		final Iterator<Attribute> attrIterator = clss.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		final Attribute attr3 = attrIterator.next();
		final Attribute attr4 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		assertEquals(1, supertypesAttr1.size());
		final Iterator<Type> superTypesAttr1Iterator = supertypesAttr1.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr1Iterator.next()));
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		assertEquals(2, supertypesAttr2.size());
		final Iterator<Type> superTypesAttr2Iterator = supertypesAttr2.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr2Iterator.next()));
		assertEquals(attr1Type, superTypesAttr2Iterator.next());
		
		assertEquals(0, ((SumType) attr3.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr3 =
				((SumType) ((SumType) attr3.getAttrType()).getPrototype()).getSuperTypes();
		assertEquals(2, supertypesAttr3.size());
		final Iterator<Type> superTypesAttr3Iterator = supertypesAttr3.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr3Iterator.next()));
		assertEquals(attr1Type, superTypesAttr3Iterator.next());
		
		assertEquals(0, ((SumType) attr4.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr4 =
				((SumType) ((SumType) attr4.getAttrType()).getPrototype()).getSuperTypes();
		assertEquals(2, supertypesAttr4.size());
		final Iterator<Type> superTypesAttr4Iterator = supertypesAttr4.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr4Iterator.next()));
		assertEquals(attr1Type, superTypesAttr4Iterator.next());
		
		assertEquals(4, clss.getSuperTypes().size());
		assertEquals(1, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{B,C};}; B:class=A+{}; C:class=A+{};
	 * Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testSumSupertypeIsAnClass() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{B,C};}; "
						+ "B:class=A+{}; C:class=A+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		final Iterator<Attribute> attrIterator = clss.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		assertEquals(1, supertypesAttr1.size());
		final Iterator<Type> superTypesAttr1Iterator = supertypesAttr1.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr1Iterator.next()));
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		
		final Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(attr1Type);
		expectedSupertypes.add(model.getAnything().getPrototype());
		expectedSupertypes.add(clss);
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr2);
		
		assertEquals(2, clss.getSuperTypes().size());
		assertEquals(2, model.getString().getSuperTypes().size());
		assertEquals(2, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{B,C};}; B:class=A+{}; C:class=A+{};
	 * D:class=A+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testSumSupertypeIsAnClass2() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{B,C};}; "
						+ "B:class=A+{}; C:class=A+{}; D:class=A+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		final Iterator<Attribute> attrIterator = clss.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		assertEquals(1, supertypesAttr1.size());
		final Iterator<Type> superTypesAttr1Iterator = supertypesAttr1.iterator();
		final Type supertype = superTypesAttr1Iterator.next();
		final ByReferenceState state =
				ByReferenceState.create(
						model.getAnything().getPrototype(),
						UnqualifiedName.create(IdentifierToken.create(
								BaseTypeConstants.ANYTHING_CONSTANT,
								DummyToken.getDummyPosition())));
		final TypeProxy typeProxy = TypeProxy.create(DummyToken.getInstance(), state, DummyToken.getInstance());
		assertEquals(typeProxy, supertype);
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(supertype));
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		
		final Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(clss);
		expectedSupertypes.add(attr1Type);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr2);
		
		assertEquals(2, clss.getSuperTypes().size());
		assertEquals(2, model.getString().getSuperTypes().size());
		assertEquals(2, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={ Attribute1:{B,C};}; B:class=A+{}; C:class=A+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testSumSupertypeIsAnClass3() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={ Attribute1:{B,C};}; "
						+ "B:class=A+{}; C:class=A+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		final Iterator<Attribute> attrIterator = clss.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		assertEquals(2, supertypesAttr1.size());
		
		final Set<Type> expectedTypes = new HashSet<>();
		ByReferenceState state =
				ByReferenceState.create(
						model.getAnything().getPrototype(),
						UnqualifiedName.create(IdentifierToken.create(
								BaseTypeConstants.ANYTHING_CONSTANT,
								DummyToken.getDummyPosition())));
		expectedTypes.add(TypeProxy.create(DummyToken.getInstance(), state, DummyToken.getInstance()));
		state = ByReferenceState.create(clss.getPrototype(), clss.getName());
		expectedTypes.add(TypeProxy.create(DummyToken.getInstance(), state, DummyToken.getInstance()));
		
		final Set<Type> actualTypes = new HashSet<>(supertypesAttr1);
		assertEquals(expectedTypes, actualTypes);
		
		assertEquals(1, clss.getSuperTypes().size());
		assertEquals(1, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{String,A}; Attribute3:{B,C};}; B:class=A+{};
	 * C:class=A+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testTransitiveInheritance() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={Attribute1:{String,Integer,A}; Attribute2:{String,A}; "
						+ "Attribute3:{B,C};}; B:class=A+{}; C:class=A+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final ClassType clss = (ClassType) model.getGroups().iterator().next().getGroupElements().iterator().next();
		final Iterator<Attribute> attrIterator = clss.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		final Attribute attr3 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		assertEquals(1, supertypesAttr1.size());
		final Iterator<Type> superTypesAttr1Iterator = supertypesAttr1.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr1Iterator.next()));
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Type attr2Type = ((SumType) attr2.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		
		Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(model.getAnything().getPrototype());
		expectedSupertypes.add(attr1Type);
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr2);
		
		assertEquals(0, ((SumType) attr3.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr3 =
				((SumType) ((SumType) attr3.getAttrType()).getPrototype()).getSuperTypes();
		
		expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(attr1Type);
		expectedSupertypes.add(attr2Type);
		expectedSupertypes.add(clss);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr3);
		
		assertEquals(3, clss.getSuperTypes().size());
		assertEquals(3, model.getString().getSuperTypes().size());
		assertEquals(2, model.getInteger().getSuperTypes().size());
		
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{}; Attribute2:{String,A}; Attribute3:{Integer,String};}; B:class=A+{};
	 * C:class=A+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testEmptySum() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(" Group:group=[A:class={Attribute1:{}; Attribute2:{String,A}; "
						+ "Attribute3:{Integer,String};}; B:class=A+{}; C:class=A+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final Iterator<GroupElement> iterator = model.getGroups().iterator().next().getGroupElements().iterator();
		
		final ClassType clssA = (ClassType) iterator.next();
		final ClassType clssB = (ClassType) iterator.next();
		final ClassType clssC = (ClassType) iterator.next();
		final ClassType clssClass = (ClassType) iterator.next();
		
		final Iterator<Attribute> attrIterator = clssA.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		final Attribute attr3 = attrIterator.next();
		
		final Type attr2Type = ((SumType) attr2.getAttrType()).getPrototype();
		final Type attr3Type = ((SumType) attr3.getAttrType()).getPrototype();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		
		final Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(clssA);
		expectedSupertypes.add(clssB);
		expectedSupertypes.add(clssC);
		expectedSupertypes.add(clssClass);
		expectedSupertypes.add(attr3Type);
		expectedSupertypes.add(attr2Type);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr1);
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		assertEquals(1, supertypesAttr2.size());
		final Iterator<Type> superTypesAttr2Iterator = supertypesAttr2.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr2Iterator.next()));
		
		assertEquals(0, ((SumType) attr3.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr3 =
				((SumType) ((SumType) attr3.getAttrType()).getPrototype()).getSuperTypes();
		assertEquals(1, supertypesAttr3.size());
		final Iterator<Type> superTypesAttr3Iterator = supertypesAttr3.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr3Iterator.next()));
		
		assertEquals(2, clssA.getSuperTypes().size());
		assertEquals(1, clssB.getSuperTypes().size());
		assertEquals(1, clssC.getSuperTypes().size());
		assertEquals(3, model.getString().getSuperTypes().size());
		assertEquals(2, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{}; Attribute2:{String,A,Anything}; Attribute3:{Integer,String};}; B:class=A+{};
	 * C:class=A+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testEmptySum2() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(" Group:group=[A:class={Attribute1:{}; Attribute2:{String,A,Anything}; "
						+ "Attribute3:{Integer,String};}; B:class=A+{}; C:class=A+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final Iterator<GroupElement> iterator = model.getGroups().iterator().next().getGroupElements().iterator();
		
		final ClassType clssA = (ClassType) iterator.next();
		final ClassType clssB = (ClassType) iterator.next();
		final ClassType clssC = (ClassType) iterator.next();
		final ClassType clssClass = (ClassType) iterator.next();
		
		final Iterator<Attribute> attrIterator = clssA.getAttributes().iterator();
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		final Attribute attr3 = attrIterator.next();
		
		final Type attr3Type = ((SumType) attr3.getAttrType()).getPrototype();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		
		final Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(clssA);
		expectedSupertypes.add(clssB);
		expectedSupertypes.add(clssC);
		expectedSupertypes.add(clssClass);
		expectedSupertypes.add(attr3Type);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr1);
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr2 =
				((SumType) ((SumType) attr2.getAttrType()).getPrototype()).getSuperTypes();
		assertEquals(0, supertypesAttr2.size());
		
		assertEquals(0, ((SumType) attr3.getAttrType()).getSuperTypes().size());
		final Collection<Type> supertypesAttr3 =
				((SumType) ((SumType) attr3.getAttrType()).getPrototype()).getSuperTypes();
		assertEquals(1, supertypesAttr3.size());
		final Iterator<Type> superTypesAttr3Iterator = supertypesAttr3.iterator();
		assertEquals(model.getAnything().getPrototype(), HelperUtils.getReferencedType(superTypesAttr3Iterator.next()));
		
		assertEquals(1, clssA.getSuperTypes().size());
		assertEquals(1, clssB.getSuperTypes().size());
		assertEquals(1, clssC.getSuperTypes().size());
		// String < {Integer,String}, String < Anything (= {String,A,Anything} !)
		assertEquals(2, model.getString().getSuperTypes().size());
		// Integer < {Integer,String}, Integer < Anything
		assertEquals(2, model.getInteger().getSuperTypes().size());
		
		final ByReferenceState state =
				ByReferenceState.create(
						model.getAnything().getPrototype(),
						UnqualifiedName.create(IdentifierToken.create(
								BaseTypeConstants.ANYTHING_CONSTANT,
								DummyToken.getDummyPosition())));
		final TypeProxy typeProxy = TypeProxy.create(DummyToken.getInstance(), state, DummyToken.getInstance());
		assertEquals(typeProxy, model.getInteger().getSuperTypes().iterator().next());
		assertEquals(
				model.getAnything().getPrototype(),
				HelperUtils.getReferencedType(model.getInteger().getSuperTypes().iterator().next()));
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{C,D};}; B:class=A+{}; C:class=B+{}; D:class=B+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testTransitiveInheritanceAndSumSupertypeIsClass() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={Attribute1:{C,D};}; B:class=A+{}; C:class=B+{}; "
						+ "D:class=B+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final Iterator<GroupElement> iterator = model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType clssA = (ClassType) iterator.next();
		final ClassType clssB = (ClassType) iterator.next();
		
		final Iterator<Attribute> attrIterator = clssA.getAttributes().iterator();
		
		final Attribute attr1 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		
		final Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(clssA);
		expectedSupertypes.add(clssB);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr1);
		
		assertEquals(1, clssA.getSuperTypes().size());
		assertEquals(1, clssB.getSuperTypes().size());
		assertEquals(1, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{B,C}; Attribute2:{D,E,F,G};}; B:class=A+{}; C:class=A+{}; D:class=B+{};
	 * E:class=B+{}; F:class=C+{}; G:class=C+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testTransitiveInheritanceAndSumSupertypeIsClass2() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={Attribute1:{B,C}; Attribute2:{D,E,F,G};}; B:class=A+{}; "
						+ "C:class=A+{}; D:class=B+{}; E:class=B+{}; F:class=C+{}; G:class=C+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final Iterator<GroupElement> iterator = model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType clssA = (ClassType) iterator.next();
		
		final Iterator<Attribute> attrIterator = clssA.getAttributes().iterator();
		
		final Attribute attr1 = attrIterator.next();
		final Attribute attr2 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		
		Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(clssA);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr1);
		
		assertEquals(0, ((SumType) attr2.getAttrType()).getSuperTypes().size());
		final Type attr2Type = ((SumType) attr2.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr2 = ((SumType) attr2Type).getSuperTypes();
		
		expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(clssA);
		expectedSupertypes.add(attr1Type);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr2);
		
		assertEquals(1, clssA.getSuperTypes().size());
		assertEquals(1, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{C,D};}; B:class=A+{}; C:class=B+{}; D:class=B+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testTransitiveInheritanceAndSumSupertypeIsClass3() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={Attribute1:{C,D};}; B:class=A+{}; C:class=B+{}; "
						+ "D:class=B+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final Iterator<GroupElement> iterator = model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType clssA = (ClassType) iterator.next();
		final ClassType clssB = (ClassType) iterator.next();
		
		final Iterator<Attribute> attrIterator = clssA.getAttributes().iterator();
		
		final Attribute attr1 = attrIterator.next();
		
		assertEquals(0, ((SumType) attr1.getAttrType()).getSuperTypes().size());
		final Type attr1Type = ((SumType) attr1.getAttrType()).getPrototype();
		final Collection<Type> supertypesAttr1 = ((SumType) attr1Type).getSuperTypes();
		
		final Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(clssB);
		expectedSupertypes.add(clssA);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr1);
		
		assertEquals(1, clssA.getSuperTypes().size());
		assertEquals(1, clssB.getSuperTypes().size());
		assertEquals(1, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
		
	}
	
	/**
	 * Group:group=[A:class={Attribute1:(name:String, alter: Integer, b:B, clss:Class);}; B:class=A+{}; C:class=B+{};
	 * D:class=B+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testProductHierarchie() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[A:class={Attribute1:(name:String, alter: Integer, b:B, clss:Class);}; B:class=A+{}; C:class=B+{}; "
								+ "D:class=B+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final Iterator<GroupElement> iterator = model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType clssA = (ClassType) iterator.next();
		final ClassType clssB = (ClassType) iterator.next();
		iterator.next();
		iterator.next();
		final ClassType clss = (ClassType) iterator.next();
		
		final Iterator<Attribute> attrIterator = clssA.getAttributes().iterator();
		
		final Attribute attr1 = attrIterator.next();
		
		assertEquals(0, attr1.getAttrType().getSuperTypes().size());
		final Type attr1Type = attr1.getAttrType().getPrototype();
		final Collection<Type> supertypesAttr1 = attr1Type.getSuperTypes();
		
		Collection<Type> expectedSupertypes = new ArrayList<>();
		final ProductType abstrProductCreated = ProductType.create(DummyToken.getInstance(), DummyToken.getInstance());
		ByReferenceState state = ByReferenceState.create(model.getString(), model.getString().getTypeName());
		TypeProxy typeProxy = TypeProxy.create(DummyToken.getInstance(), state);
		abstrProductCreated.addElement(ProductElementType.create("p$1", typeProxy, DummyToken.getInstance()));
		state = ByReferenceState.create(model.getInteger(), model.getInteger().getTypeName());
		typeProxy = TypeProxy.create(DummyToken.getInstance(), state);
		abstrProductCreated.addElement(ProductElementType.create("p$2", typeProxy, DummyToken.getInstance()));
		state = ByReferenceState.create(clssB, clssB.getName());
		typeProxy = TypeProxy.create(DummyToken.getInstance(), state);
		abstrProductCreated.addElement(ProductElementType.create("p$3", typeProxy, DummyToken.getInstance()));
		final ByReferenceState state2 = ByReferenceState.create(clss, clss.getName());
		final TypeProxy typeProxy2 = TypeProxy.create(DummyToken.getInstance(), state2);
		abstrProductCreated.addElement(ProductElementType.create("p$4", typeProxy2, DummyToken.getInstance()));
		expectedSupertypes.add(model.getAnything().getPrototype());
		expectedSupertypes.add(abstrProductCreated);
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr1);
		
		ProductType abstrProduct = null;
		for (final Type t : supertypesAttr1) {
			if (t.equals(abstrProductCreated)) {
				abstrProduct = (ProductType) t;
				break;
			}
		}
		assertNotNull(abstrProduct);
		expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, abstrProduct.getSuperTypes());
		
		assertEquals(1, clssA.getSuperTypes().size());
		assertEquals(1, clssB.getSuperTypes().size());
		assertEquals(1, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
		
	}
	
	/**
	 * Group:group=[A:class={Attribute1:(p1:String, p2: Integer, p3:B, p4:Class);}; B:class=A+{}; C:class=B+{};
	 * D:class=B+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testProductHierarchie2() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[A:class={Attribute1:(p1:String, p2: Integer, p3:B, p4:Class);}; B:class=A+{}; C:class=B+{}; "
								+ "D:class=B+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final Iterator<GroupElement> iterator = model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType clssA = (ClassType) iterator.next();
		final ClassType clssB = (ClassType) iterator.next();
		
		final Iterator<Attribute> attrIterator = clssA.getAttributes().iterator();
		
		final Attribute attr1 = attrIterator.next();
		
		assertEquals(0, attr1.getAttrType().getSuperTypes().size());
		final Type attr1Type = attr1.getAttrType().getPrototype();
		final Collection<Type> supertypesAttr1 = attr1Type.getSuperTypes();
		
		final Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(model.getAnything().getPrototype());
		expectedSupertypes.add(((ProductType) attr1Type).getAbstractPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr1);
		
		assertEquals(1, clssA.getSuperTypes().size());
		assertEquals(1, clssB.getSuperTypes().size());
		assertEquals(1, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
		
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{(name:String, alter: Integer, b:B, clss:Class),String};}; B:class=A+{};
	 * C:class=B+{}; D:class=B+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testProductHierarchie3() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[A:class={Attribute1:{(name:String, alter: Integer, b:B, clss:Class),String};}; B:class=A+{}; C:class=B+{}; "
								+ "D:class=B+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final Iterator<GroupElement> iterator = model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType clssA = (ClassType) iterator.next();
		final ClassType clssB = (ClassType) iterator.next();
		iterator.next();
		iterator.next();
		final ClassType clss = (ClassType) iterator.next();
		
		final Iterator<Attribute> attrIterator = clssA.getAttributes().iterator();
		
		final Attribute attr1 = attrIterator.next();
		
		// Summe
		assertEquals(0, attr1.getAttrType().getSuperTypes().size());
		final Type attr1Type = attr1.getAttrType().getPrototype();
		final Collection<Type> supertypesAttr1 = attr1Type.getSuperTypes();
		
		Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr1);
		
		// Product
		final SumType sum = (SumType) attr1Type;
		final Iterator<Type> iterator2 = sum.getElements().iterator();
		iterator2.next();
		final ProductType productPrototyp = (ProductType) iterator2.next();
		
		expectedSupertypes = new ArrayList<>();
		final ProductType abstrProductCreated = ProductType.create(DummyToken.getInstance(), DummyToken.getInstance());
		ByReferenceState state = ByReferenceState.create(model.getString(), model.getString().getTypeName());
		TypeProxy typeProxy = TypeProxy.create(DummyToken.getInstance(), state);
		abstrProductCreated.addElement(ProductElementType.create("p$1", typeProxy, DummyToken.getInstance()));
		state = ByReferenceState.create(model.getInteger(), model.getInteger().getTypeName());
		typeProxy = TypeProxy.create(DummyToken.getInstance(), state);
		abstrProductCreated.addElement(ProductElementType.create("p$2", typeProxy, DummyToken.getInstance()));
		state = ByReferenceState.create(clssB, clssB.getName());
		typeProxy = TypeProxy.create(DummyToken.getInstance(), state);
		abstrProductCreated.addElement(ProductElementType.create("p$3", typeProxy, DummyToken.getInstance()));
		final ByReferenceState state2 = ByReferenceState.create(clss, clss.getName());
		final TypeProxy typeProxy2 = TypeProxy.create(DummyToken.getInstance(), state2);
		abstrProductCreated.addElement(ProductElementType.create("p$4", typeProxy2, DummyToken.getInstance()));
		expectedSupertypes.add(abstrProductCreated);
		expectedSupertypes.add(sum);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, productPrototyp.getSuperTypes());
		
		assertEquals(1, clssA.getSuperTypes().size());
		assertEquals(2, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Group:group=[A:class={Attribute1:{(name:String, alter: Integer, b:B, clss:Class),(p1:String, p2: Integer, p3:B,
	 * p4:Class)};}; B:class=A+{}; C:class=B+{}; D:class=B+{}; Class:class={};]; .
	 * 
	 * @throws Exception
	 *             Exception
	 */
	@Test
	public void testProductHierarchie4() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[A:class={Attribute1:{(name:String, alter: Integer, b:B, clss:Class),(p1:String, p2: Integer, p3:B, p4:Class)};}; B:class=A+{}; C:class=B+{}; "
								+ "D:class=B+{}; Class:class={};];");
		final Model model = this.getModel(input);
		
		assertEquals(0, model.getAnything().getSuperTypes().size());
		assertEquals(0, ((SumType) model.getAnything().getPrototype()).getSuperTypes().size());
		
		final Iterator<GroupElement> iterator = model.getGroups().iterator().next().getGroupElements().iterator();
		final ClassType clssA = (ClassType) iterator.next();
		final ClassType clssB = (ClassType) iterator.next();
		iterator.next();
		iterator.next();
		final ClassType clss = (ClassType) iterator.next();
		
		final Iterator<Attribute> attrIterator = clssA.getAttributes().iterator();
		
		final Attribute attr1 = attrIterator.next();
		
		// Summe
		assertEquals(0, attr1.getAttrType().getSuperTypes().size());
		final Type attr1Type = attr1.getAttrType().getPrototype();
		final Collection<Type> supertypesAttr1 = attr1Type.getSuperTypes();
		
		Collection<Type> expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, supertypesAttr1);
		
		// Produkte
		final SumType sum = (SumType) attr1Type;
		final Iterator<Type> iterator2 = sum.getElements().iterator();
		final ProductType productPrototyp = (ProductType) iterator2.next();
		final ProductType productPrototyp2 = (ProductType) iterator2.next();
		
		expectedSupertypes = new ArrayList<>();
		final ProductType abstrProductCreated = ProductType.create(DummyToken.getInstance(), DummyToken.getInstance());
		ByReferenceState state = ByReferenceState.create(model.getString(), model.getString().getTypeName());
		TypeProxy typeProxy = TypeProxy.create(DummyToken.getInstance(), state);
		abstrProductCreated.addElement(ProductElementType.create("p$1", typeProxy, DummyToken.getInstance()));
		state = ByReferenceState.create(model.getInteger(), model.getInteger().getTypeName());
		typeProxy = TypeProxy.create(DummyToken.getInstance(), state);
		abstrProductCreated.addElement(ProductElementType.create("p$2", typeProxy, DummyToken.getInstance()));
		state = ByReferenceState.create(clssB, clssB.getName());
		typeProxy = TypeProxy.create(DummyToken.getInstance(), state);
		abstrProductCreated.addElement(ProductElementType.create("p$3", typeProxy, DummyToken.getInstance()));
		final ByReferenceState state2 = ByReferenceState.create(clss, clss.getName());
		final TypeProxy typeProxy2 = TypeProxy.create(DummyToken.getInstance(), state2);
		abstrProductCreated.addElement(ProductElementType.create("p$4", typeProxy2, DummyToken.getInstance()));
		expectedSupertypes.add(abstrProductCreated);
		expectedSupertypes.add(sum);
		expectedSupertypes.add(model.getAnything().getPrototype());
		
		this.checkSupertypes(expectedSupertypes, productPrototyp.getSuperTypes());
		
		expectedSupertypes = new ArrayList<>();
		expectedSupertypes.add(sum);
		expectedSupertypes.add(model.getAnything().getPrototype());
		expectedSupertypes.add(productPrototyp2.getAbstractPrototype());
		
		this.checkSupertypes(expectedSupertypes, productPrototyp2.getSuperTypes());
		
		assertEquals(1, clssA.getSuperTypes().size());
		assertEquals(1, model.getString().getSuperTypes().size());
		assertEquals(1, model.getInteger().getSuperTypes().size());
	}
	
	/**
	 * Assert Equality of expectedSupertypes and supertypes.
	 * 
	 * @param expectedSupertypes
	 *            expectedSupertypes
	 * @param supertypes
	 *            supertypes
	 */
	private void checkSupertypes(final Collection<Type> expectedSupertypes, final Collection<Type> supertypes) {
		assertEquals(expectedSupertypes.size(), supertypes.size());
		for (final Type type : supertypes) {
			assertTrue(expectedSupertypes.remove(HelperUtils.getReferencedType(type)));
		}
	}
}
